'use server';

import Docker from 'dockerode';
import fs from 'fs/promises'; // 使用异步版本的 fs 模块
import path from 'path';
import axios from 'axios';
import dockerConfig from './dockerConfig.json';

const docker = new Docker();

/**
 * 执行多文件代码并测试结果
 * @param params 包含代码文件列表、语言、测试数据文件名和正确答案文件名的参数
 * @returns 运行结果或比较结果
 */
export const runMultiFileCodeWithOptionalTestData = async (params: {
    files: { name: string; content: string }[];
    language: string;
    testDataFiles?: string[];
    expectedAnswerFiles?: string[];
}) => {
    const { files, language, testDataFiles, expectedAnswerFiles } = params;

    if (!files || files.length === 0 || !language) {
        throw new Error('Files and language are required.');
    }

    const supportedLanguages = ['c', 'java', 'python'];
    if (!supportedLanguages.includes(language)) {
        throw new Error(`Language '${language}' is not supported. Only 'c', 'java', and 'python' are supported.`);
    }

    const compileAndRunCmd =
        language === 'c'
            ? 'gcc *.c -o main && ./main'
            : language === 'java'
                ? 'javac *.java && java Main'
                : 'python3 script.py';

    const tempDir = path.join('/tmp', `${Date.now()}`);
    await fs.mkdir(tempDir, { recursive: true });

    try {
        // 写入代码文件
        await Promise.all(files.map(async (file) => {
            const filePath = path.join(tempDir, file.name);
            await fs.writeFile(filePath, file.content);
        }));

        // 创建 Docker 容器
        const container = await docker.createContainer({
            ...dockerConfig,
            Cmd: ['sh', '-c', compileAndRunCmd],
            HostConfig: {
                Binds: [`${tempDir}:/workspace`], // 挂载临时目录到容器
            },
        });

        await container.start();

        const logs = await container.logs({ stdout: true, stderr: true, follow: true });

        let output = '';
        logs.on('data', (data: Buffer) => {
            output += data.toString().replace(/[\x00-\x1F\x7F]/g, ''); // 去除不可见字符
        });

        const exitCode = await container.wait();
        await container.remove();

        if (exitCode.StatusCode !== 0) {
            return { error: `Execution failed with exit code ${exitCode.StatusCode}. Output: ${output}` };
        }

        // 检查结果输出文件是否存在
        const resultOutputFile = dockerConfig.ResultOutputFile;
        const resultFilePath = path.join(tempDir, resultOutputFile);

        try {
            const resultFileContent = await fs.readFile(resultFilePath, 'utf-8');
            return { output, resultFileContent };
        } catch (error) {
            // 如果结果文件不存在，返回控制台输出
            return { output };
        }
    } catch (error) {
        return { error: `An error occurred: ${(error as Error).message}` };
    } finally {
        await fs.rm(tempDir, { recursive: true, force: true });
    }
};
